package com.slideme.demo.model.slidelock;

import android.app.Activity;
import android.os.AsyncTask;
import android.util.Log;
import com.slideme.slidelock.License;

import java.io.File;
import java.io.IOException;


/**
 * Sample Asynchronous Task that retrieves a binary license from SlideME License Servers
 * and stores it on filesystem for reuse at a later time.
 * Overwrite method onPostExecute(Rights result) to handle license verification.
 */
public class LicenseCheckAsyncTask extends AsyncTask<Void, Void, License.Rights> {

    protected final String licenseURI;
    protected final License license;

    public LicenseCheckAsyncTask(Activity anActivity, String slideLockKey) {
        this(anActivity.getFilesDir().getPath() + File.separator + Integer.toHexString(slideLockKey.hashCode()) + ".bin",
                new License(slideLockKey, anActivity));
    }

    public LicenseCheckAsyncTask(String licenseURI, License license) {
        this.licenseURI = licenseURI;
        this.license = license;
    }

    @Override
    protected License.Rights doInBackground(Void... voids) {
        byte[] licenseBinary = null;
        try {
            Log.d("LicenseCheckAsyncTask", "Trying to restore license from FS");
            licenseBinary = LicenseUtils.retrieveLicense(licenseURI);
        } catch (IOException e) {
            Log.w("LicenseCheckAsyncTask", "Could not restore from FS", e);
        }
        if (licenseBinary != null && licenseBinary.length > 0)
            try {
                Log.d("LicenseCheckAsyncTask", "Digesting restored license from FS");
                license.digest(licenseBinary);
            } catch (IOException ignored) {
            }

        if (license.getFullRights() == null) {
            try {
                Log.d("LicenseCheckAsyncTask", "Fetching license from SlideME Servers");
                licenseBinary = license.fetch();
                if (licenseBinary != null && licenseBinary.length > 0) {
                    license.digest(licenseBinary);
                    if (license.getFullRights() != null) { /* Only store license when valid */
                        Log.d("LicenseCheckAsyncTask", "Storing license on FS");
                        LicenseUtils.storeLicense(licenseURI, licenseBinary);
                    }
                } else {
                    Log.d("LicenseCheckAsyncTask", "Received invalid license");
                }
            } catch (IOException e) {
                Log.d("LicenseCheckAsyncTask", "Unexpected IOE", e);
            }
        }
        return license.getFullRights();
    }


}
